#SkIndigo Ruby code is copyright Dale Martens 2009, all rights reserved, and
#Copyright Glare Technologies Limited 2014 -

#TODO
#make work with </null_material>

module DMxml

class DMxml::Element

	attr_reader :children, :name, :parent
	attr_writer :parent, :element_text, :name

	#this class should contain an array of all child elements and the text contained by this element
	def initialize(name)
		@name = name
		@parent = nil
		@children = []  # An array of other elements
		@element_text = ""
	end

	###add an element to the document
	def add_child(element)
		@children.push(element)
	end

	#######
	def start_tag
		return "<#{@name}>"
	end

	######
	def end_tag
		return "<\/#{@name}>"
	end

	#####
	def to_s(depth=0)
		# if there are no children, just write the tag on line line, e.g. "	<name>bleh</name>"
		if children.empty?
			return ("\t" * depth) + start_tag + @element_text + end_tag + "\n"
		else
			xml =  ("\t" * depth) + start_tag + "\n"
			xml += ("\t" * depth) + @element_text + "\n" if !@element_text.empty?
			children.each do |e|
				xml += e.to_s(depth + 1)
			end
			xml += ("\t" * depth) + end_tag + "\n"
			return xml
		end
	end

	######
	def [](child_name)
		return @children.find {|e| e.name==child_name}
	end

	#######
	def find_all(child_name)
		return @children.find_all {|e| e.name==child_name}
	end

	###
	def strip_cdata(string)
		mod_string = string
		if string =~ /<!\[CDATA\[/
			mod_string = string.split("<![CDATA[")[1].split("]]>")[0]
		end

		return mod_string
	end

	####
	def element_text
		return strip_cdata(@element_text)
	end

end  #end element class



############
class DMxml::Document

	def initialize
		@children = []   #the top-level elements in the document
		#@xml = ""
		@all_elements = []  #an array of all elements in the document
	end

	###add an element to the document
	def add_child(element)
		@children.push(element)
	end

	def add_element(element)
		@all_elements.push(element)
	end

	########
	def children
		return @children
	end
	
	# Returns the root element
	def root
		return @children[0]
	end

	###########
	def to_s(depth = 0)
		#self.children_to_s(self, depth + 1)
		#return @xml
		xml = ""
		children.each do |e|
			xml += e.to_s(depth)
		end
		return xml
	end

	#######
	def find_element(element_name)
		@all_elements.find {|e| e.name==element_name}
	end

	######
	def [](child_name)
		child = @children.find {|e| e.name==child_name}
		return child
	end

end #end class



class DMxml::Tag

	attr_accessor :name, :type

	def initialize(name,type)
		@name = name
		@type = type  #0 for start, 1 for end, 2 for special
	end

end  #end class



##############
class DMxml::Parser

	def initialize(doc)
		@document = doc  #the string representation of the document
		@xml_doc = DMxml::Document.new()
		@parent = [@xml_doc]  #will store the stack of parent elements
		@index = 0   #the current character index in the document
		@current_element = nil
		@start_text_index = 0   #keep track of the index of the text in the current element
		@end_text_index = 0 
	end

	#####
	def get_next_tag(string)
		#p string
		#p "current index #{@index}"
		tag_positions = []
		start_re = /<\w+>/   #reg expression representing a start tag
		end_re = /<\/\w+>/  #reg expression representing an end tag
		sp_re = /<\w+\s*\/>/  #reg expression representing a special tag such as <null_material/> 
		pos_start = string =~ start_re   #find the next start tag in the string
		if pos_start
			tag_positions.push(pos_start)
			start_tag_name=$&[1..-2]
			start_tag_text=$&
			#p "found start tag #{start_tag_text}"
			s_tag=DMxml::Tag.new(start_tag_name,0)
		end
		pos_end=string =~ end_re  #find the next end tag in the string
		if pos_end
			tag_positions.push(pos_end)
			end_tag_name=$&[2..-2]
			end_tag_text=$&
			#p "found end tag #{end_tag_text}"
			e_tag=DMxml::Tag.new(end_tag_name,1)
		end
		pos_sp=string =~ sp_re  #find the next special tag
		if pos_sp
			tag_positions.push(pos_sp)
			sp_tag_name=$&[1..-3]
			sp_tag_text=$&
			#p "found special tag #{sp_tag_text}"
			sp_tag=DMxml::Tag.new(sp_tag_name,2)
		end

		min_pos = tag_positions.min  #get the minimum tag position
		case min_pos
		when nil
			return nil
		when pos_start
			#p pos_start
			@index=@index+pos_start+start_tag_text.length
			@start_text_index=@index
			#p "start tag #{start_tag_text} returned"
			return s_tag #return the start tag name if it occurrd first
		when pos_end
			#p pos_start
			#p pos_end
			@index=@index+pos_end+end_tag_text.length
			#p "end tag #{end_tag_text} returned"
			return e_tag
		when pos_sp
			@index=@index+pos_sp+sp_tag_text.length
			return sp_tag
		end

	end  #end get_next_tag

	
	###parse the document and build the tree
	def parse_document()   #doc is a string representing the entire xml file
		parse_string(@document)
		return @xml_doc
	end

	#####parse a string
	def parse_string(string)

		next_tag = get_next_tag(string)  #get the next tag in the string
		while next_tag
			case next_tag.type
			when 0  #start tag
				#create a new element object
				#p "found start tag #{next_tag.name} at index #{@index}"
				new_element = DMxml::Element.new(next_tag.name)
				new_element.parent = @parent.last
				@parent.last.add_child(new_element)
				@parent.push(new_element)
				@xml_doc.add_element(new_element)
			when 1 #end tag
				current_element = @parent.pop
				#p "found end tag #{next_tag.name} at index #{@index}"
				if (current_element.name==next_tag.name) and (current_element.children.length==0)
					#get the text between the end tag and the start tag
					current_element_text=@document[@start_text_index..(@index-(current_element.end_tag.length+1))]
					#p "element named #{next_tag.name} has text #{current_element_text}"
					current_element.element_text=current_element_text.strip  #set the element text and strip leading and trailing white space
					#current_element.element_text=strip_cdata(current_element.element_text) #removes cdata tag
				end
			when 2 #special tag
				new_element = DMxml::Element.new(next_tag.name)
				new_element.parent = @parent.last
				@parent.last.add_child(new_element)
				@xml_doc.add_element(new_element)
				
			end #end case
		
			#string=string[@index..-1]
			next_tag = get_next_tag(string[@index..-1])

		end  #end while loop

	end  #end parse string

end #class end

end #end module
