#SkIndigo Ruby code is copyright Dale Martens 2011, all rights reserved, and
#Glare Technologies Limited, 2014 - 
#-----------------------------------------------------------------------------

#######################################SkIndigo Material Editor############################3



#TEMP: copied from skindigo_core.rb
module SkIndigo
module Dialog

	#this function clears web-dialog dropdown list by calling a javascript function that must be defined in the webdialog
	def clear_dropdown(dlg,id)

		cmd="clearDropDown('#{id}')"
		dlg.execute_script(cmd)
	end

	#this function creates an option to be added to a dropdown list by calling a javascript function that must be defined inside the webdialog
	def create_option(dlg,id,name,value)

		cmd="createMapOption('#{id}','#{name}','#{value}')"
		dlg.execute_script(cmd)
	end

end #module
end #module


def reinterpret_as_utf8(str)
	if String.method_defined?(:encode)
		str = str.encode('UTF-16', 'UTF-8', :invalid => :replace, :replace => '') # Convert to UTF-16
		str = str.encode('UTF-8', 'UTF-16') # Convert back to UTF-8
	end
	return str
end


class SkindigoMatObserver <  Sketchup::MaterialsObserver

	include SkIndigoUVS
	include SkIndigo::Dialog

	def initialize
		@model=Sketchup.active_model
		@default_ui=Sketchup.read_default("SkIndigoDefaults",'ui',"normal")
		@dict="indigo07_settings"
		@ui="normal" # Force normal material UI mode.
		@on_mac=SkIndigo.on_mac?
		@preview_process_id=nil
		preview_folder="su_preview_scenes"
		@custom_preview_scene_path=File.join(SkIndigo.get_indigo_app_data_path,preview_folder)
		Dir.mkdir(@custom_preview_scene_path) unless FileTest.exist?(@custom_preview_scene_path)
		@standard_preview_scene_path=File.join(File.dirname(reinterpret_as_utf8(__FILE__)),"SUpreviews")
		@start=nil
		@preset_dlg=nil
		@mdb_dlg=nil
		mats=Sketchup.active_model.materials
		#initialize the currently selected material
		if mats.length>0
			current_mat=mats.current
			if current_mat==nil
				mats.current=mats[0]  #select the first material
			end
			@sm=IndigoMaterial.new(mats.current)
		else  #no materials found so create one
			new_mat=mats.add("Material1")
			new_mat.color=[125,125,125]
			mats.current=new_mat
			@sm=IndigoMaterial.new(new_mat)
		end
		#@selection_stack=[Sketchup.active_model.materials.current]
		@selection_stack=[]
		@mat_editor=UI::WebDialog.new("SkIndigo Material Editor", false, "SkIndigoMaterialEditor",307, 750, 1,1,true)
		plugins=Sketchup.find_support_file("Plugins\/skindigo")
		fn=File.join(plugins,"MaterialEditor.html")
		@mat_editor.set_file fn
		color = @mat_editor.get_default_dialog_color 
		@mat_editor.set_background_color(color)
		@mat_editor.set_on_close {@preset_dlg.close if (@preset_dlg) and (@preset_dlg.visible?)}
		#############Primary Event Handler Callback Function for Material Editor##################

		@mat_editor.add_action_callback("ValueChanged") {|d,p|
			pair=p.split("=")
			id=pair[0]           
			value=pair[1]
			#p "#{id} changed to #{value}"
			#check if the selected material is still valid (not deleted)
			unless sm.mat.valid?
				UI.messagebox("Material not found.")
				#select the first material
				self.sm=IndigoMaterial.new(Sketchup.active_model.materials[0])
				refresh_dialog()
				return
			end
			
			type=sm.type
			case id
				when "AssignPresetButton"
					
					if @preset_dlg
						@preset_dlg.show{} unless @preset_dlg.visible?
						@preset_dlg.bring_to_front
						return
					end
						
					#Create the Preset Selection Dialog
					@preset_dlg=UI::WebDialog.new("Select Material Preset", 
						true, # scrollable
						"SkIndigoPresets", 300, 768, 272, 1, true)
					fn=File.join(File.dirname(reinterpret_as_utf8(__FILE__)),"presets.html")
					@preset_dlg.set_file fn
					color = @preset_dlg.get_default_dialog_color 
					@preset_dlg.set_background_color(color)
					@preset_dlg.add_action_callback("assign_preset") {|d,p|
						params=p.split("=")
						preset=params[0]
						use_thumb=params[1]
						if use_thumb=="true"
							new_thumbnail=File.join(File.dirname(reinterpret_as_utf8(__FILE__)),preset+".jpg")
							sm.thumbnail=new_thumbnail
						end
						case preset
							when "glass"
								apply_glass(sm)
							when "frostedGlass"
								apply_frosted_glass(sm)
							when "mirror"
								apply_mirror(sm)
							when "ceramic"
								apply_ceramic(sm)
							when "dullPlastic"
								apply_dull_plastic(sm)
							when "shinyPlastic"
								apply_shiny_plastic(sm)
							when "leather"
								apply_leather(sm)
							when "carPaint"
								apply_car_paint(sm)
							when "lampshade"
								apply_lampshade(sm)
							when "gold"
								apply_nkdata(sm,"Au.nk")
							when "tintedGlass"
								apply_tinted_glass(sm)
							when "water"
								apply_water(sm)
							when "lambertian"
								apply_lambertian(sm)
							when "clay"
								apply_clay(sm)
							when "white_wax"
								apply_wax(sm)
							when "orangejuice"
								apply_oj(sm)
							when "incan100w"
								apply_100W(sm)
							when "fluor34wcool"
								apply_34WTFC(sm)
							when "fluor34wmid"
								apply_34WTFM(sm)
							when "fluor34wwarm"
								apply_34WTFW(sm)
						end
						loadValuesFromSU()   #updates the material editor with new material values
						setVisibility()
							
					}
						
					@preset_dlg.show{}
						
				when "SearchMDB"
					
					if @mdb_dlg
						fn=File.join(File.dirname(reinterpret_as_utf8(__FILE__)),"mdbsearch.html")
						@mdb_dlg.set_file fn
						@mdb_dlg.show{} unless @mdb_dlg.visible?
						@mdb_dlg.bring_to_front
						return
					end
						
					#Create the Material DB search dialog
					@mdb_dlg=UI::WebDialog.new("Search Indigo Material Database", false,"SkIndigoMaterialSearch", 530, 768, 272, 1, true)
					fn=File.join(File.dirname(reinterpret_as_utf8(__FILE__)),"mdbsearch.html")
					@mdb_dlg.set_file fn
					color = @mdb_dlg.get_default_dialog_color 
					@mdb_dlg.set_background_color(color)
					@mdb_dlg.add_action_callback("download") {|d,p|
						result=UI.messagebox("Download and load into selected material?",MB_YESNO,"Download material")
						if result==6
							begin
								igm=download_igm(p)
								#p "download completed"
								status=nil
								if igm!=nil
									f_name=File.basename(igm)
									if f_name.split(".")[-1]=="pigm"  #loading a PIGM file
										igm=SkIndigo.unpack_pigm(igm)
										#return if igm==nil
										Dir.chdir(File.dirname(igm))  #set the working directory to the unpacked igm file
									end
									imi=IndigoMaterialImporter.new(igm)
									status=imi.load_igm(igm,sm)
									if status==0
										UI.messagebox("Material successfully loaded.")
									elsif status==1
										sm.type="Linked IGM"
										self.create_linked_igm(igm)
									end
									refresh_dialog()
								end
								
							rescue
								UI.messagebox("Material could not be loaded.")
								raise
							end
						elsif result==7
							igm_path=UI.savepanel("Save Material","",File.basename(p))
							if igm_path!=nil
								igm=download_igm(p,igm_path)
								if FileTest.exist?(igm)
									UI.messagebox("Material saved to #{igm}.")
								else
									UI.messagebox("Material could not be saved to #{igm}.  Plese check read/write permissions for this folder.")
								end
							end
						
						end #if result
							
						#UI.openURL(p)
					}
					@mdb_dlg.add_action_callback("search_mdb") {|d,p|
						@mdb_dlg.execute_script("searchMDB();")
					}
					@mdb_dlg.show{}
				when "SwitchUI"
					if @ui=="normal" 				 
						@ui=@model.set_attribute(@dict,'ui',"simple")
						@mat_editor.set_size(300,600)
					else	
						@ui=@model.set_attribute(@dict,'ui',"normal")
						@mat_editor.set_size(300,768)
					end
					
					setVisibility()
					loadValuesFromSU()
				when "sampleMaterial"
					Sketchup.active_model.select_tool SkIndigoMaterialPicker.new
				when "previewScene"
					if value=="New"
						status=create_preview_scene()
					end
					if status
						refresh_dialog()
					end
				when "PreviewButton"		
					if ((type=="Blackbody Light") or (type=="RGB Light") or (type=="Exit Portal"))
						UI.messagebox "#{type} materials are currently not supported by this feature."
					else
						path=SkIndigo.get_indigo_path
						return if not path
						thumbnail_path=IndigoRenderSettings.new().thumbnail_path
						if not FileTest.exist?(thumbnail_path)
							UI.messagebox "Thumbnail path not found. Preview aborted."
							return
						end
						mat_name = SkIndigo.sanitise_string(sm.mat.name)
						new_thumbnail = File.join(thumbnail_path, mat_name + ".png")
						File.delete(new_thumbnail) if FileTest.exist?(new_thumbnail)
						sm.thumbnail = new_thumbnail
						if @preview_process_id
						#we are currently rendering a preview so stop it before starting a new one
							stop_preview()
						end
						@preview_process_id = preview_material(sm, value, new_thumbnail)
						if @preview_process_id
							toggle_stop_button(d,true)
							if @timer
								UI.stop_timer(@timer)
							end
							@timer=UI.start_timer(1, true) do
								getThumbnail()
							end # Update material preview every 1 second.
						end						 
					end
				when "StopPreview"
					self.stop_preview()
				when "SaveIGMButton"
					if ((type=="Blackbody Light") or (type=="RGB Light") or (type=="Exit Portal") or (type=="Linked IGM"))
						UI.messagebox "#{type} materials are currently not supported by this feature."
				
					else
						igm=UI.savepanel("Export IGM/PIGM", "",((sm.name.delete("<>"))+".igm"))
						return if igm==nil
						file_name=File.basename(igm,".*")
						igm=File.join(File.dirname(igm),file_name+".igm")
						result=UI.messagebox("Would you like to pack this IGM? (Recommended for materials with textures)",MB_YESNO)
						if result==6
							pack=true
						else
							pack=false
						end		
						IndigoExporter.new(igm).export_igm(sm,pack)
					end
				when "LoadIGMButton"
						if SkIndigo.sketchup_is_2014_or_later # Multiple file type wildcards were broken before SketchUp 2014: http://www.sketchup.com/intl/en/developer/docs/ourdoc/ui?csw=1#openpanel
							igm = UI.openpanel("Import IGM/PIGM", "", "Indigo material file (*.igm;*.pigm)|*.igm;*.pigm||")
						else
							igm = UI.openpanel("Import IGM/PIGM", "", "Indigo material file (.igm;.pigm)")
						end
						
						return if igm==nil
						status=nil
						if (FileTest.exist?(igm))
							f_name=File.basename(igm)
							if f_name.split(".")[-1]=="pigm"  #loading a PIGM file
								#if SkIndigo.on_mac?
									#UI.messagebox("PIGM Loading is not yet available on Mac systems.  You must first unpack/unzip the PIGM using another application (eg. Stuffit).  Then, load the IGM file.")
									#return
								#end
								igm=SkIndigo.unpack_pigm(igm)
								return if igm==nil
								Dir.chdir(File.dirname(igm))  #set the working directory to the unpacked igm file
							end
							#result=load_igm(igm,sm)
							#TODO error reporting
							imi=IndigoMaterialImporter.new(igm)
							status=imi.load_igm(igm,sm)
							if status==0
								UI.messagebox("Material succesfully loaded.")
							elsif status==1
								sm.type="Linked IGM"
								self.create_linked_igm(igm)
							end			
						end
						refresh_dialog()
					
				when "editIGMButton"
					if SkIndigo.on_mac?
						UI.messagebox("This feature is not yet available on Mac systems.")
					else
						igm=d.get_element_value("igm")
						if FileTest.exist?(igm)
							UI.openURL("file:///"+File.expand_path(igm))
						else
							UI.messagebox("IGM path not valid.")
						end
					end
				when "MaterialName"
					new_selected_mat=IndigoMaterial.find(value)
					if new_selected_mat
						self.sm=new_selected_mat
						Sketchup.active_model.materials.current=self.sm.mat  #set the current material in the Sketchup Materials Window
						refresh_dialog()
						@mat_editor.bring_to_front #ensure the focus stays on the material editor and does not switch to the Sketchup Materials window
					else
						UI.messagebox("Unable to find material named #{value}")
					end
				when "type"
					sm.type=value
					sm.alpha=0.2 if value=="Thin Glass"
					if value=="Specular" or value=="Glossy Transparent"
						if sm.alpha==1.0 and sm.absorption_map.type=="SketchUp"
							UI.messagebox("Warning: SketchUp opacity is currently set to 100%.  If you wish to create a transparent material, lower the opacity setting to near zero using the SketchUp material editor.")
						end
					end
					setVisibility()
				when "layer"
					sm.layer=value
				when "ior"
					sm.ior=value
				when "archGlassCheckbox"
					sm.arch_glass=true if value=="true"
					sm.arch_glass=false if value=="false"
				when "cau"
					sm.cauchy_b_coeff=value
				when "exp"
					sm.exponent=value
				when "precedence"
					sm.precedence=value
				when "temp"
					sm.temperature=value
				when "power"
					sm.power_drawn=value
				when "ole"
					sm.overall_luminous_efficacy=value
				when "g_spectrum"
					sm.g_spectrum=value
				when "sssval"
					sm.sss_spectrum=value
				when "fres"
					sm.fresnel_scale=value
				when "sigma"
					sm.sigma=value
				when "subSmoothingCheckbox"
					sm.smooth=true if value=="true"
					sm.smooth=false if value=="false"
				when "viewDependentCheckbox"
					sm.view_dependent_subdivision=true if value=="true"
					sm.view_dependent_subdivision=false if value=="false"
				when "EnableIESCheckbox"
					sm.ies=true if value=="true"
					sm.ies=false if value=="false"
				when "SSSCheckbox"
					sm.sss=true if value=="true"
					sm.sss=false if value=="false"
				when "bumpCheckbox"
					sm.use_bump=true if value=="true"
					sm.use_bump=false if value=="false"
				when "normalCheckbox"
					sm.use_normal=true if value=="true"
					sm.use_normal=false if value=="false"
				when "displaceCheckbox"
					sm.use_displace=true if value=="true"
					sm.use_displace=false if value=="false"
				when "emissionCheckbox"
					sm.use_emission=true if value=="true"
					sm.use_emission=false if value=="false"
				when "absLayerTransCheckbox"
					sm.use_abs_layer_trans=true if value=="true"
					sm.use_abs_layer_trans=false if value=="false"
				when "clipCheckbox"
					sm.use_clip=true if value=="true"
					sm.use_clip=false if value=="false"
				when "stepBlend"
					sm.step_blend=true if value=="true"
					sm.step_blend=false if value=="false"
				when "interferenceCheckbox"
					sm.interference=true if value=="true"
					sm.interference=false if value=="false"
				when "specularCheckbox"
					if value=="true"
						sm.use_specular=true
						sm.use_nkdata=false
						setCheckbox("EnableNKdataCheckbox","false")
					else 
						sm.use_specular=false
					 end				
				when "EnableNKdataCheckbox"
					if value=="true"
						sm.use_nkdata=true
						sm.use_specular=false
						setCheckbox("specularCheckbox","false")
					else
						sm.use_nkdata=false
					end
				when "Phase_Function"
					sm.phase_function=value
					if value=="Uniform"
						setVisible("g_spectrum_param",false)
					else
						setVisible("g_spectrum_param",true)
					end
				#when "UniformPF"
				#	sm.phase_function="Uniform"
				#when "Henyey-GreensteinPF"
				#	sm.phase_function="Henyey-Greenstein"
				when "bbES" #blackbody emission spectrum
					sm.base_emission_spectrum="Blackbody"
				when "rgbES" #rgb emission specrum
					sm.base_emission_spectrum="RGB"
					
				when "albedo_map"
					tex=sm.albedo_texture
					tex.type=value
					sm.albedo_texture=tex
					setColorBox("albedoSwatch",sm.color("albedo"))
				when "specular_map"
					tex=sm.specular_map
					tex.type=value
					sm.specular_map=tex
					setColorBox("specularSwatch",sm.color("specular"))
				when "blend_map"
					tex=sm.blend_map
					tex.type=value
					sm.blend_map=tex
					setValueBoxVisible(tex,"blendfactor")
				when "bump_map"
					tex=sm.bump_map   #get the current bump map
					tex.type=value        #assign the SU material name to the texture object
					sm.bump_map=tex   #assign the updated texture object back to the selected material
					setValueBoxVisible(tex,"bump_value")
				when "normal_map"
					tex=sm.normal_map   
					tex.type=value        
					sm.normal_map=tex   
					setValueBoxVisible(tex,"normalmap_value")
				when "exp_map"
					tex=sm.exponent_map   
					tex.type=value       
					sm.exponent_map=tex
					setValueBoxVisible(tex,"expmap_value")
				when "clip_map"
					tex=sm.clip_map   
					tex.type=value       
					sm.clip_map=tex
					setValueBoxVisible(tex,"clipValue")
				when "disp_map"
					tex=sm.disp_map
					tex.type=value
					sm.disp_map=tex
					setValueBoxVisible(tex,"dispmap_value")
				when "emission_map"
					tex=sm.emission_map
					tex.type=value
					sm.emission_map=tex
					setColorBox("emissionSwatch",sm.color("emission"))
				when "transmittance_map"
					tex=sm.transmittance_map
					tex.type=value
					sm.transmittance_map=tex
					setColorBox("transmittanceSwatch",sm.color("transmittance"))
				when "abs_layer_trans_map"
					tex=sm.abs_layer_trans_map
					tex.type=value
					sm.abs_layer_trans_map=tex
					setColorBox("abslayertransSwatch",sm.color("absorption_layer_transmittance"))
				when "sigma_map"
					tex=sm.sigma_map
					tex.type=value
					sm.sigma_map=tex
					setValueBoxVisible(tex,"sigma_value")
				when "roughness_map"
					tex=sm.roughness_map
					tex.type=value
					sm.roughness_map=tex
					setValueBoxVisible(tex,"roughness_value")
				when "fresnel_map"
					tex=sm.fresnel_map
					tex.type=value
					sm.fresnel_map=tex
					setValueBoxVisible(tex,"fresnel_value")
				when "thickness_map"
					tex=sm.thickness_map
					tex.type=value
					sm.thickness_map=tex
					setValueBoxVisible(tex,"thickness_value")
				when "front_roughness_map"
					tex=sm.front_roughness_map
					tex.type=value
					sm.front_roughness_map=tex
					setValueBoxVisible(tex,"front_roughness_value")
				when "back_roughness_map"
					tex=sm.back_roughness_map
					tex.type=value
					sm.back_roughness_map=tex
					setValueBoxVisible(tex,"back_roughness_value")
				when "front_fresnel_map"
					tex=sm.front_fresnel_scale_map
					tex.type=value
					sm.front_fresnel_scale_map=tex
					setValueBoxVisible(tex,"front_fresnel_value")
				when "back_fresnel_map"
					tex=sm.back_fresnel_scale_map
					tex.type=value
					sm.back_fresnel_scale_map=tex
					setValueBoxVisible(tex,"back_fresnel_value")
				when "r_f_map"
					tex=sm.r_f_map
					tex.type=value
					sm.r_f_map=tex
					setValueBoxVisible(tex,"r_f__value")
				when "abs_map"
					tex=sm.absorption_map
					tex.type=value
					sm.absorption_map=tex
					setColorBox("absorption",sm.color("absorption"))
				when "sss_map"
					tex=sm.sss_map
					tex.type=value
					sm.sss_map=tex
					setColorBox("sssSwatch",sm.color("sss"))
				when "bump_value"
					tex=sm.bump_map   
					tex.b=value       
					sm.bump_map=tex
				when "normalmap_value"
					tex=sm.normal_map   
					tex.b=value       
					sm.normal_map=tex
				when "expmap_value"
					tex=sm.exponent_map   
					tex.b=value       
					sm.exponent_map=tex
				when "dispmap_value"
					tex=sm.disp_map
					tex.b=value
					sm.disp_map=tex
				when "emission_value"
					tex=sm.emission_map
					tex.b=value
					sm.emission_map=tex
				when "abs_layer_trans_value"
					tex=sm.abs_layer_trans_map
					tex.b=value
					sm.abs_layer_trans_map=tex
				when "sigma_value"
					tex=sm.sigma_map
					tex.b=value
					sm.sigma_map=tex
				when "roughness_value"
					tex=sm.roughness_map
					tex.b=value
					sm.roughness_map=tex
				when "front_roughness_value"
					tex=sm.front_roughness_map
					tex.b=value
					sm.front_roughness_map=tex
				when "back_roughness_value"
					tex=sm.back_roughness_map
					tex.b=value
					sm.back_roughness_map=tex
				when "front_fresnel_value"
					tex=sm.front_fresnel_scale_map
					tex.b=value
					sm.front_fresnel_scale_map=tex
				when "back_fresnel_value"
					tex=sm.back_fresnel_scale_map
					tex.b=value
					sm.back_fresnel_scale_map=tex
				when "r_f_value"
					tex=sm.r_f_map
					tex.b=value
					sm.r_f_map=tex
				when "fresnel_value"
					tex=sm.fresnel_map
					tex.b=value
					sm.fresnel_map=tex
				when "thickness_value"
					tex=sm.thickness_map
					tex.b=value.to_f/1000000.0
					p tex.b
					sm.thickness_map=tex
				when "clipValue"
					tex=sm.clip_map
					tex.b=value
					sm.clip_map=tex
				when "advancedAlbedoMap","advancedEmissionMap","advancedAbsLayerTransMap","advancedSpecularMap","advancedAbsMap","advancedSSSMap","advancedG_spectrumMap","advancedBaseEmissionMap","advancedTransmittanceMap"
					params=[sm.albedo_texture,"albedo",0.0,1.0] if id=="advancedAlbedoMap"
					params=[sm.emission_map,"emission",0.0,1.0] if id=="advancedEmissionMap"
					params=[sm.abs_layer_trans_map,"abs_layer_trans",0.0,1.0] if id=="advancedAbsLayerTransMap"
					params=[sm.specular_map,"specular",0.0,1.0] if id=="advancedSpecularMap"
					params=[sm.absorption_map,"absorption",0.0,nil] if id=="advancedAbsMap"
					params=[sm.sss_map,"sss",0.0,nil] if id=="advancedSSSMap"
					params=[sm.g_spectrum_map,"g_spectrum",-1.0,1.0] if id=="advancedG_spectrumMap"
					params=[sm.base_emission,"base_emission",0.0,nil] if id=="advancedBaseEmissionMap"
					params=[sm.transmittance_map,"transmittance",0.0,1.0] if id=="advancedTransmittanceMap"
					case params[0].type
					when "Constant"
					ite=IndigoSpectrumPicker.new(params[0],sm,params[2],params[3],params[1])
					when "External","SketchUp"
						if params[0].type=="SketchUp" and params[1]=="absorption"
							UI.messagebox("Use the SketchUp color and opacity settings to modify the absorption.")
						else
							ite=IndigoTextureEditor.new(params[0],sm,params[1])
						end
					when "Shader"
					ite=IndigoShaderEditor.new(params[0],sm,params[1])
					end
					
				when "advancedBumpMap","advancedNormalMap","advancedDispMap","advancedExponentMap","advancedClipMap","advancedBlendMap","advancedSigmaMap","advancedRoughnessMap","advancedFresnelMap","advancedThicknessMap","advancedr_fMap","advancedFrontFresnelMap","advancedBackFresnelMap","advancedFrontRoughnessMap","advancedBackRoughnessMap"
					params=[sm.bump_map,"bump"] if id=="advancedBumpMap"
					params=[sm.disp_map,"displacement"] if id=="advancedDispMap"
					params=[sm.exponent_map,"exponent"] if id=="advancedExponentMap"
					params=[sm.clip_map,"clip"] if id=="advancedClipMap"
					params=[sm.blend_map,"blend"] if id=="advancedBlendMap"
					params=[sm.sigma_map,"sigma"] if id=="advancedSigmaMap"
					params=[sm.roughness_map,"roughness"] if id=="advancedRoughnessMap"
					params=[sm.fresnel_map,"fresnel"] if id=="advancedFresnelMap"
					params=[sm.thickness_map,"thickness"] if id=="advancedThicknessMap"
					params=[sm.normal_map,"normal"] if id=="advancedNormalMap"
					params=[sm.r_f_map,"r_f"] if id=="advancedr_fMap"
					params=[sm.front_fresnel_scale_map,"front fresnel"] if id=="advancedFrontFresnelMap"
					params=[sm.back_fresnel_scale_map,"back fresnel"] if id=="advancedBackFresnelMap"
					params=[sm.front_roughness_map,"front roughness"] if id=="advancedFrontRoughnessMap"
					params=[sm.back_roughness_map,"back roughness"] if id=="advancedBackRoughnessMap"
					case params[0].type
					when "Constant" #constant
						UI.messagebox("There are no additional parameters for a 'constant' #{params[1]} attribute.")
					when "External","SketchUp"
						ite=IndigoTextureEditor.new(params[0],sm,params[1])
					when "Shader"
						ite=IndigoShaderEditor.new(params[0],sm,params[1])
					end
				when "matA"
					sm.a_name=value
				when "matB"
					sm.b_name=value
				when "substrate"
					sm.substrate=value
				when "front_material_name"
					sm.set(id,value)
				when "back_material_name"
					sm.set(id,value)
				when "blendfactor"
					#sm.blend_factor=value
					tex=sm.blend_map
					tex.b=value
					sm.blend_map=tex
				when "nkdata"
					p value
					sm.nk_data=NKdata.new(value)
				when "ies_path"
					sm.ies_profile=IESProfile.new(value)
				when "metalsList"
					#root=File.dirname(SkIndigo.get_indigo_path) unless @on_mac
					#root=SkIndigo.get_indigo_app_data_path if @on_mac
					
					#nkdata=File.join(root,"nkdata",value+".nk")
					nkdata=File.join("nkdata",value+".nk")
					nk=NKdata.new(nkdata)
					sm.nk_data=nk
					sm.use_nkdata=true
					sm.use_specular=false
					setCheckbox("specularCheckbox","false")
					setCheckbox("EnableNKdataCheckbox","true")
					setValue("nkdata",nk.to_s)
				when "nkdataBrowse"
					UI.messagebox("All NKdata files must be located in the Indigo Renderer\/nkdata folder.")
					path = UI.openpanel "Load NKdata", "", "*.nk"
					if path
						path=File.join("nkdata",File.basename(path))
						nkdata=NKdata.new(path)
						sm.nk_data=nkdata
						setValue("nkdata",nkdata.to_s)
					end
				when "iesBrowse"
					path=UI.openpanel "Load IES Profile", "", "*.ies"
					if path
						ies=IESProfile.new(path)
						sm.ies_profile=ies
						setValue("ies_path",ies)
					end
				when "igmBrowseButton"
					self.create_linked_igm(nil)
				when "slider-1"  #reflection slider
					if (type!="RGB Light") and (type!="Blackbody Light")
						type=getTypeFromSliders()
						sm.type=type
					end
					if value=="0"
						ior="1.0"
					elsif value=="100"
						ior="1000"
					else
						ior=to_ior(value)
					end
					sm.ior=ior
				when "slider-2" #roughness slider
					#p "roughness modified"
					type=sm.type
					if (type!="RGB Light") and (type!="Blackbody Light")
						type=getTypeFromSliders()
						sm.type=type
					end
					if ((type=="Phong") or (type=="Glossy Transparent"))
						exp=to_exp(value)
						sm.exponent=exp
					end
				when "slider-3" #power slider
					power=to_power(value)
					#sm.power_drawn=power
					sm.emission_scale=power.to_i*100
				when "emitterCheckbox"
					if value=="true"
						sm.layer="0"  #set the layer to 0 to make the material an emitter
						be=sm.base_emission
						be.spectrum_type="Blackbody"
						be.bb_temperature=6500
						sm.base_emission=be
						em=sm.emission_map
						em.type="SketchUp"
						sm.use_emission=true
						sm.emission_map=em
						#setCheckbox("transparentCheckbox","false")
					else
						sm.layer="-1"
						sm.use_emission=false
						type=getTypeFromSliders()
						sm.type=type
					end
				when "transparentCheckbox"
					rough=d.get_element_value("roughness-value")
					if value=="true"
						if rough=="0"
							sm.type="Specular"
						else
							sm.type="Glossy Transparent"
						end
						#setCheckbox("emitterCheckbox","false")
					else
						if rough=="100"
							sm.type="Diffuse"
						else
							sm.type="Phong"
						end
					end
				when "emissionScale"
					sm.emission_scale=value
				when "emissionMeasure"
					sm.emission_measure=value
				when "maxSubdiv"
					sm.max_subdiv=value
				when "pixelThreshold"
					sm.pixel_threshold=value
				when "curveThreshold"
					sm.curvature_threshold=value
				when "matAButton"
					mata=d.get_element_value("matA")
					if mata!="None"
						@selection_stack.push(self.sm)
						self.sm=IndigoMaterial.find(mata)
						Sketchup.active_model.materials.current=sm.mat
						loadValuesFromSU()
						setVisibility()
					end
				when "matBButton"
					matb=d.get_element_value("matB")
					if matb!="None"
						@selection_stack.push(self.sm)
						self.sm=IndigoMaterial.find(matb)
						Sketchup.active_model.materials.current=sm.mat
						loadValuesFromSU()
						setVisibility()
					end
				when "substrateButton"
					substrate=d.get_element_value("substrate")
					if substrate!="None"
						@selection_stack.push(self.sm)
						self.sm=IndigoMaterial.find(substrate)
						Sketchup.active_model.materials.current=sm.mat
						loadValuesFromSU()
						setVisibility()
					end
				when "frontMatButton"
					front_material_name=d.get_element_value("front_material_name")
					if front_material_name!="None"
						@selection_stack.push(self.sm)
						self.sm=IndigoMaterial.find(front_material_name)
						Sketchup.active_model.materials.current=sm.mat
						loadValuesFromSU()
						setVisibility()
					end
				when "backMatButton"
					back_material_name=d.get_element_value("back_material_name")
					if back_material_name!="None"
						@selection_stack.push(self.sm)
						self.sm=IndigoMaterial.find(back_material_name)
						Sketchup.active_model.materials.current=sm.mat
						loadValuesFromSU()
						setVisibility()
					end
				when "backButton"
					valid=false
					#p @selection_stack
					while not valid
						unless @selection_stack.empty?
							self.sm=@selection_stack.pop()
							if self.sm.valid? 
								valid=true
								Sketchup.active_model.materials.current=sm.mat
							end
						else
							UI.messagebox("You cannot go back unless you click on the Material A or B buttons of a Blend material.")
							valid=true
						end
					end
					loadValuesFromSU()
					setVisibility()
				when "baseEmissionColorBox"
					#cp=ColorPicker.new(sm.base_emission,sm,"base_emission")
					#cp.show
					#current=Sketchup.active_model.materials.current
					#if current
					#	cp=IndigoMaterial.new(current)
					#	if cp.color_picker?  #if the color picker is the current material
					#		col=cp.color.to_a
					#		sm.base_emission=[col[0],col[1],col[2]]
					#		setColorBox(id,[col[0],col[1],col[2]])
					#	else
					#		old_col=sm.base_emission
					#		cp=IndigoColorPicker.new(old_col)
					#	end
					#end
				when "baseEmissionGain"
					sm.base_emission_gain=value.to_f
				when "VOID_absorption"
					prompts=["R (m^-1)","G (m^-1)","B (m^-1)"]
					values=sm.absorption_spectrum
					results=UI.inputbox(prompts,values,"Absorption RGB Spectrum")
					if results
						sm.absorption_spectrum=results.join(" ")
						self.refresh_dialog()
					end
				#spectrum picker teset
					
			end #case id

			ite=nil   #indigo texture editor
			
			} #end of Valuechanged callback function

		@mat_editor.add_action_callback("RefreshPreview") {|d,p| getThumbnail}
		@mat_editor.add_action_callback("SetVisibility") {|d,p| setVisibility()}
		@mat_editor.add_action_callback("showThumbnail"){|d,p|
			thumb_dlg=UI::WebDialog.new("SkIndigo Thumbnail", false,"SkIndigoThumbnail", 295, 320, 450, 125, true)
			thumb_dlg.set_html "<img src=\"#{p}\" style=\"cursor:pointer\" title=\"Double-click to load custom thumbnail\"ondblclick=\"window.location='skp:loadThumbnail';\">"
			thumb_dlg.add_action_callback("loadThumbnail") {|d,p|
				new_thumb = SkIndigo.openpanel_sketchup_supported_image("Load Thumbnail")
				if ((new_thumb) and (FileTest.exist?(new_thumb)))
					sm.thumbnail=new_thumb
					getThumbnail()
				end}
			thumb_dlg.show{}
		}
		@mat_editor.add_action_callback("RefreshValues") {|d,p| 
				#p "refreshing values"
				refresh_dialog()
				
		}
		@mat_editor.add_action_callback("check_current_version") {|d,p|
			version=p.to_s
			current=SkIndigo::VERSION.to_s
			if version>current
				text="A newer version of SkIndigo is available. Would you like to update?  (Click 'No' to be reminded in two weeks.)"
				result=UI.messagebox(text,MB_YESNO)
				if result==6 #yes
					UI.openURL("http://www.indigorenderer.com/documentation/sketchup")
				else
					time=Time.now.to_i
					Sketchup.write_default("SkIndigo","LastUpdateCheck",time)
				end
			end
		}
		@mat_editor.set_on_close{ self.stop_preview() }
			
	end #initialize

	##
	def create_linked_igm(path=nil)

		if path==nil
			if SkIndigo.sketchup_is_2014_or_later # Multiple file type wildcards were broken before SketchUp 2014: http://www.sketchup.com/intl/en/developer/docs/ourdoc/ui?csw=1#openpanel
				path = UI.openpanel("Link IGM/PIGM", "", "Indigo material file (*.igm;*.pigm)|*.igm;*.pigm||")
			else
				path = UI.openpanel("Link IGM/PIGM", "", "Indigo material file (.igm;.pigm)")
			end
		end

		igm=nil
		if path
			name=File.basename(path)
			if name.split(".")[-1]=="pigm"
				igm=SkIndigo.unpack_pigm(path)
				if igm==nil  #problems unpacking file
					UI.messagebox("PIGM file could not be unpacked.")
					return
				end
			elsif name.split(".")[-1]=="igm"
				igm=path
			end
		end
		if igm
			igm=igm.gsub(/\\/,"\/")
			sm.linked_igm=igm
			setValue("igm",igm)
			result=UI.messagebox("Would you like to apply a new UV texture to this material?",MB_YESNO)
			if result==6  #yes
				#igm_dir=(File.dirname(igm)).gsub(/\//,"\\")
				tex_path = SkIndigo.openpanel_sketchup_supported_image("Load texture")
				if tex_path
					sm.texture=tex_path  #set the SU texture to the new texture
				end
			end
		end
	end

	#####
	def check_for_updates()

		last_update_check=Sketchup.read_default("SkIndigo","LastUpdateCheck",0)
		current_time=Time.now.to_i
		update_check_period=1209600

		if current_time-last_update_check>update_check_period  #check for updates every 2 weeks
			@mat_editor.execute_script("check_current_version();")
		end
	end


	def sm
		@sm
	end

	def timer
		@timer
	end
		
	def sm=(new_mat)
		#p "newly selected material is #{new_mat.name}"
		@sm=new_mat
	end

	def refreshMaterialsList()

		mat_list=Sketchup.active_model.materials.collect {|m| m.name}
		mat_list.sort!

		options = mat_list.map { |mat_name| [mat_name, mat_name] }
		
		SkIndigo.set_dropdown_options(@mat_editor, "MaterialName", options)
	end

	def refreshTextureList()

		#function disabled - this was used for previous version internal textures
		return

	end

	def refreshBlendList()

		blend_list = getBlendList()
		
		options = blend_list.map { |tex| [tex, tex] }
		
		SkIndigo.set_dropdown_options(@mat_editor, "matA", options)
		SkIndigo.set_dropdown_options(@mat_editor, "matB", options)
	end


	#generic dropdown list refreshing function that should be used for refreshing all dropdowns going forward
	def refreshList(id,list)

		options = list.map { |item| [item, item] }
		
		SkIndigo.set_dropdown_options(@mat_editor, id, options)
	end

	def refreshLayerList()
		cmd="clearDropDown('layer')"
		@mat_editor.execute_script(cmd)
		layer_names=IndigoRenderSettings.new.light_layer_names
		cmd="createMapOption('layer','Off','-1')"
		@mat_editor.execute_script(cmd)
		layer_names.each_index  {|i|
			cmd="createMapOption('layer',\"#{layer_names[i]}\",\"#{i}\")"
			@mat_editor.execute_script(cmd)
		}
	end

	def refreshPreviewSceneList()
		clear_dropdown(@mat_editor,"previewScene")
		scenes=get_preview_scenes(@standard_preview_scene_path)
		scenes.each {|name,path| create_option(@mat_editor,"previewScene",name,path) }
		scenes=get_preview_scenes(@custom_preview_scene_path)
		scenes.each {|name,path| create_option(@mat_editor,"previewScene",name,path) }
		create_option(@mat_editor,"previewScene","New..","New")
	end

	#returns the list of preview scenes as an array of pairs of type [Display name, path to preview scene]
	def get_preview_scenes(path)

		begin
			# Names on disk, and 'nice' names to display in the UI.
			nice_names = { "mat_db" => "Mat DB", "mat_db1m" => "Mat DB 1 m", "mat_db10m" => "Mat DB 10 m" }

			path=path.gsub(/\\/,"\/")
			scenes=Dir.glob(path + "/**/*.igs")
			
			scenes=scenes.find_all {|f| File.basename(f)[0..2]=="IMP"}

			scene_hash={}
			scenes.each {|scene|
				bn=File.basename(scene,".igs")
				bn.slice!("IMP")  #remove the IMP from the name
				
				# Replace with a 'nice name' if present
				bn = nice_names[bn] if nice_names[bn]
				
				scene_hash[bn]=scene
			}

			# Turn the hash into an array, and then sort it. (alphabetically)
			res_array = scene_hash.to_a.sort()
			return res_array
		rescue Exception => e
			# Failed to open the dir or somesuch
			puts "Exception raised in get_preview_scenes():"
			puts e
			return []
		end
	end


	##create a new preview scene
	def create_preview_scene()

		begin
		mats=Sketchup.active_model.materials
		pm=mats["previewmaterial"]
		if pm==nil
			UI.messagebox("You must have a material named 'previewmaterial' in your model and it must be applied to at least one face.")
		end
		result=UI.inputbox(["Name"],[""],"Enter Scene Name")
		return nil if not result
		scene_name=result[0]
		preview_scene=File.join(@custom_preview_scene_path,"IMP"+scene_name+".igs")
		if FileTest.exist?(preview_scene)
			result=UI.messagebox("Overwrite existing scene?",MB_OKCANCEL)
			if result==1 #OK
				File.delete(preview_scene)
			else
				return false
			end
		end
		export_path=SkIndigo.temp_dir
		if not FileTest.exist?(export_path)
			Dir.mkdir(export_path)
		end
		export_path=File.join(export_path,scene_name+".igs")
		status=SkIndigo.export(:mat_preview,true,export_path,false,false)
		if status
			#pack the scene
			console=SkIndigo.get_indigo_console_path
			temp_pigs=File.join(SkIndigo.temp_dir,"IMP"+scene_name+".pigs")
			params="--pack \"#{export_path}\" \"#{temp_pigs}\""
			SkIndigo.launch_indigo(console,"",params,true,"hidden")
			try=0
			while !FileTest.exist?(temp_pigs)
				raise "Packing Timed Out" if try==5  #allow 10 seconds for indigo to pack the scene
				try+=1
				sleep(1)
			end
			
			#now unpack the scene to the su_preview directory
			params="--unpack \"#{temp_pigs}\" \"#{@custom_preview_scene_path}\""
			SkIndigo.launch_indigo(console,"",params,true,"hidden")
			while !FileTest.exist?(preview_scene)
				raise "Unpacking Timed Out" if try==5  #allow 10 seconds for indigo to unpack the scene
				try+=1
				sleep(1)
			end
			File.delete(temp_pigs)
			UI.messagebox("New Preview Scene #{scene_name} created.")
			return true
		end #if

		rescue
			UI.messagebox("There was a problem creating the IGS file.")
			raise
		end #begin
	end   #function

			
	def onMaterialRemove(mats, mat)
		#p "#{mat.name} was deleted."
		if mat.name==@model.get_attribute(@dict,'bg_material',"")
			new_mat=mats.add(mat.name)
			new_im=IndigoMaterial.new(new_mat)
			old_im=IndigoMaterial.new(mat)
			old_im.copy(new_im)
			@model.set_attribute(@dict,'bg_material',new_im.name)
		end
	end


	#material is changed (eg. color changed by color wheel)
	def onMaterialChange(material,skup_material)

		#basically, we are creating a delay so that the Skindigo editor will update showing the new thumbnail but there is a two second delay so that the SketchUp UI does not lag
		#due to the thumbnail texture being re-written

		if @mat_editor.visible?
			if @mat_change_timer
				UI.stop_timer(@mat_change_timer)
			end
			@mat_change_timer=UI.start_timer(2,false) {loadValuesFromSU()}
		end
	end
	
	
	# A new material was added to SketchUp.  
	#def onMaterialAdd(materials, skup_material)
		
		# When a new material gets added to SketchUp, it copies all the attributes from the last selected material by design
		# We can't mess with this.  This is a core behavior of SketchUp.  To create a fresh material with no Indigo settings
		# just select the 'default' material (the square icon with the diagonal line through it) prior to creating a new material
		#  Another reason that we can't delete the indigo attributes when adding a new material is because this method 'onMaterialAdd'
		# fires also when you import an external SKP file into your scene.  If you have intentionally pre-made Indigo materials in
		# that SKP file, then all of the Indigo materials will have their settings reset by using the method below!
		#
		#IndigoMaterial.reset_indigo_attributes(skup_material)
	#end


	#for some reason, Mac and PC Material Editors behave quite differently.  This method fires on PC when a user clicks on a material
	#in the Material Edit Window.  For Mac, you have to hold down the command button and click on a face in the model for this method to fire.
	def onMaterialSetCurrent(mats, mat)
		#p "setting current material #{mat.name}"
		#p @timer
		if @timer
			UI.stop_timer(@timer)
		end
		
		return if !mat # mat will be null if the picked material is the default material.
		
		new_mat=IndigoMaterial.new(mat)
		unless new_mat.color_picker? or new_mat.uv_helper?
			self.sm=IndigoMaterial.new(mat)
			#@selection_stack.push(self.sm)
			if @mat_editor.visible?
				refresh_dialog()
			end
			
			#testing for double-clicking the material in the paint bucket
			if @start and not @on_mac   #this doesn't work on Mac
				finish=Time.new
				if (finish-@start)<0.25     #check if user double-clicked a material in the Paint Bucket
					@mat_editor.show {refresh_dialog()} 
					#@mat_editor.bring_to_front
					#@mat_editor.show{}
				else
					@start=Time.new
				end
			else
				@start=Time.new
			end
		end

	end

	######refresh the dialog values
	def refresh_dialog()
		#t = Time.now
		#TODO: We should cache a lot of this stuff
		refreshPreviewSceneList()
		#puts "refreshPreviewSceneList: #{Time.now - t} s"; t = Time.now
		refreshMaterialsList()
		#puts "refreshMaterialsList: #{Time.now - t} s"; t = Time.now
		refreshTextureList()
		#puts "refreshTextureList: #{Time.now - t} s"; t = Time.now
		refreshBlendList()
		#puts "refreshBlendList: #{Time.now - t} s"; t = Time.now
		substrate_list = getSubstrateList()
		#puts "getSubstrateList(): #{Time.now - t} s"; t = Time.now
		refreshList("substrate", substrate_list)
		#puts "refreshList substrate: #{Time.now - t} s"; t = Time.now
		refreshList("front_material_name", substrate_list)
		#puts "refreshList front_material_name: #{Time.now - t} s"; t = Time.now
		refreshList("back_material_name", substrate_list)
		#puts "refreshList back_material_name: #{Time.now - t} s"; t = Time.now
		refreshLayerList()
		#puts "refreshLayerList: #{Time.now - t} s"; t = Time.now
		setVisibility()
		#puts "setVisibility: #{Time.now - t} s"; t = Time.now
		loadValuesFromSU()
		#puts "loadValuesFromSU: #{Time.now - t} s"; t = Time.now
		validateReferences()
		#puts "validateReferences: #{Time.now - t} s"; t = Time.now
	end

	
	def validateReferences()

		reference_ids=["advancedAlbedoMap","advancedBlendMap","advancedExponentMap","advancedSpecularMap","advancedSigmaMap","advancedRoughnessMap","advancedFresnelMap","advancedTransmittanceMap","advancedr_fMap","advancedFrontFresnelMap","advancedBackFresnelMap","advancedFrontRoughnessMap","advancedBackRoughnessMap","advancedBumpMap","advancedNormalMap","advancedDispMap","advancedEmissionMap","advancedAbsLayerTransMap","advancedClipMap","iesBrowse","nkdataBrowse","igmBrowseButton"]
		reference_ids.each {|id| validate(id)}

		invalidate("advancedAlbedoMap") if !sm.albedo_texture.valid_reference?
		invalidate("advancedBlendMap") if !sm.blend_map.valid_reference?
		invalidate("advancedExponentMap") if !sm.exponent_map.valid_reference?
		invalidate("advancedSpecularMap") if !sm.specular_map.valid_reference?
		invalidate("advancedSigmaMap") if !sm.sigma_map.valid_reference?
		invalidate("advancedBumpMap") if !sm.bump_map.valid_reference?
		invalidate("advancedNormalMap") if !sm.normal_map.valid_reference?
		invalidate("advancedDispMap") if !sm.disp_map.valid_reference?
		invalidate("advancedEmissionMap") if !sm.emission_map.valid_reference?
		invalidate("advancedAbsLayerTransMap") if !sm.abs_layer_trans_map.valid_reference?
		invalidate("advancedClipMap") if !sm.clip_map.valid_reference?
		invalidate("iesBrowse") if !sm.ies_profile.valid_reference?
		invalidate("nkdataBrowse") if !sm.nk_data.valid_reference?
		invalidate("igmBrowseButton") if !sm.valid_igm_reference?
		invalidate("advancedRoughnessMap") if !sm.roughness_map.valid_reference?
		invalidate("advancedFresnelMap") if !sm.fresnel_map.valid_reference?
		invalidate("advancedTransmittanceMap") if !sm.transmittance_map.valid_reference?
		invalidate("advancedr_fMap") if !sm.r_f_map.valid_reference?
		invalidate("advancedFrontFresnelMap") if !sm.front_fresnel_scale_map.valid_reference?
		invalidate("advancedBackFresnelMap") if !sm.back_fresnel_scale_map.valid_reference?
		invalidate("advancedFrontRoughnessMap") if !sm.front_roughness_map.valid_reference?
		invalidate("advancedBackRoughnessMap") if !sm.back_roughness_map.valid_reference?
	end

	def validate(id)

		cmd="$('##{id}').css('backgroundColor','');"
		@mat_editor.execute_script(cmd)
		cmd="$('##{id}').attr('title','');"
		@mat_editor.execute_script(cmd)
	end

	def invalidate(id)

		cmd="$('##{id}').css('backgroundColor','rgb(255,0,0)');"
		@mat_editor.execute_script(cmd)
		cmd="$('##{id}').attr('title','Invalid File Reference');"
		@mat_editor.execute_script(cmd)
	end
	
	
	####display the material editor
	def show_editor
		#p "show editor called"
		#p "selected material is #{sm.name}"
		if @timer
			UI.stop_timer(@timer)
		end
		if sm!=nil
			if @mat_editor.visible?
				#p "editor is visible"
				self.refresh_dialog()
				#refreshMaterialsList();refreshTextureList();refreshBlendList();loadValuesFromSU();setVisibility()
			else
				#p "editor is not visible"
				#@mat_editor.show{refreshMaterialsList();refreshTextureList();refreshBlendList();loadValuesFromSU();setVisibility()} if not @on_mac
				if @on_mac
					self.refresh_dialog()
					self.check_for_updates()
					@mat_editor.show_modal{}
				else
					@mat_editor.show{check_for_updates()}
				end
				#@mat_editor.show_modal{refreshMaterialsList();refreshTextureList();refreshBlendList();loadValuesFromSU();setVisibility()} if @on_mac
				#@mat_editor.show_modal{} if @on_mac  #test for bugfix
				#@mat_editor.show{}
				#@mat_editor.close
				#@mat_editor.show{}
			end
		else
			UI.messagebox "Please select a material using the Paint Bucket."
			#mats=Sketchup.active_model.materials
			#if mats.length==0  #no materials so we will create one
			#	new_mat=mats.add("Material1").color=[125,125,125]
			#	mats.current=new_mat
			#	sm=IndigoMaterial.new(new_mat)
			#else
			#	sm=IndigoMaterial.new(mats[0]) #select the first material for editing
			#end
			#self.show_editor
		end
	end
		

	def getThumbnail

		thumb=sm.thumbnail
		#p thumb
		if FileTest.exist?(thumb)
			thumb.gsub!(/\\/,"\/")
			#cmd="document.images[\"PreviewImage\"].src=\"nopreview.png\""  #This is required so that new image is not updated from the cache
			#@mat_editor.execute_script(cmd)
			cmd="document.images[\"PreviewImage\"].src=\"file:\/\/#{thumb}?#{(rand*10000).to_i}\""  #adding random data ensures the image is not loaded from the cache
			@mat_editor.execute_script(cmd)
		else
			if sm.texture!=nil
				tex_path=File.basename(sm.texture.filename)
				if tex_path  
					tex=tex_path.gsub(/\\/,"\/")
				else
					return
				end
				#p tex
				tex_path=File.join(SkIndigo.temp_dir,tex)
				#p tex_path
				tex_path=save_mat_texture(sm.mat,tex_path)
				if tex_path!=nil
					tex_path.gsub!(/\\/,"\/")
					cmd="document.images[\"PreviewImage\"].src=\"file:\/\/#{tex_path}?#{(rand*10000).to_i}\""
				end
			else
				cmd="document.images[\"PreviewImage\"].src=\"nopreview.png\""
			end
			@mat_editor.execute_script(cmd)
		end		
	end


	def loadValuesFromSU()
		#p "loading values from SU"
		getThumbnail()
		type=sm.type
		bm=sm.bump_map
		nm=sm.normal_map
		em=sm.exponent_map
		cm=sm.clip_map
		at=sm.albedo_texture
		dm=sm.disp_map
		sim=sm.sigma_map
		emm=sm.emission_map
		altm=sm.abs_layer_trans_map
		spm=sm.specular_map
		absm=sm.absorption_map
		sssm=sm.sss_map
		gsm=sm.g_spectrum_map
		rm=sm.roughness_map
		fm=sm.fresnel_map
		tm=sm.thickness_map
		transm=sm.transmittance_map
		frm=sm.front_roughness_map
		brm=sm.back_roughness_map
		ffsm=sm.front_fresnel_scale_map
		bfsm=sm.back_fresnel_scale_map
		r_fm=sm.r_f_map
		
		blm=sm.blend_map
		root=File.dirname(SkIndigo.get_indigo_path)


		###########################################Update the material editor dialog box
		setValue("MaterialName",sm.mat.name)
		setValue("type",sm.type)
		setValue("layer",sm.layer)
		setValue("ior",sm.ior)
		#setValue("exp",sm.exponent)
		#setValue("fres",sm.fresnel_scale)
		#setValue("sigma",sm.sigma)
		setValue("precedence",sm.precedence)
		setValue("cau",sm.cauchy_b_coeff)
		setValue("power",sm.power_drawn)
		setValue("ole",sm.overall_luminous_efficacy)
		###setValue("temp",sm.temperature)
		setValue("nkdata",sm.nk_data.to_s)
		setValue("ies_path",sm.ies_profile.to_s)
		###setValue("g_spectrum",sm.g_spectrum)
		###setValue("sssval",sm.sss_spectrum)
		setValue("albedo_map",at.type)
		setValue("specular_map",spm.type)
		setValue("blend_map",blm.type)
		setValue("bump_map",bm.type)
		setValue("bump_value",bm.b)
		setValue("normal_map",nm.type)
		setValue("normalmap_value",nm.b)
		setValue("disp_map",dm.type)
		setValue("dispmap_value",dm.b)
		setValue("exp_map",em.type)
		setValue("expmap_value",em.b)
		setValue("sigma_map",sim.type)
		setValue("sigma_value",sim.b)
		setValue("roughness_map",rm.type)
		setValue("roughness_value",rm.b)
		setValue("r_f_map",r_fm.type)
		setValue("r_f_value",r_fm.b)
		setValue("front_fresnel_map",ffsm.type)
		setValue("front_fresnel_value",ffsm.b)
		setValue("back_fresnel_map",bfsm.type)
		setValue("back_fresnel_value",bfsm.b)
		setValue("front_roughness_map",frm.type)
		setValue("front_roughness_value",frm.b)
		setValue("back_roughness_map",brm.type)
		setValue("back_roughness_value",brm.b)
		setValue("fresnel_map",fm.type)
		setValue("fresnel_value",fm.b)
		setValue("thickness_map",tm.type)
		setValue("thickness_value",tm.b.to_f*1000000.0)
		setValue("clipValue",cm.b)
		setValue("emission_map",emm.type)
		setValue("transmittance_map",transm.type)
		###setValue("emission_value",emm.b)
		setValue("abs_layer_trans_map",altm.type)
		###setValue("abs_layer_trans_value",altm.b)
		setValue("clip_map",cm.type)
		setValue("matA",sm.a_name)
		setValue("matB",sm.b_name)
		setValue("substrate",sm.substrate)
		setValue("front_material_name",sm.get("front_material_name"))
		setValue("back_material_name",sm.get("back_material_name"))
		#setValue("blendfactor",sm.blend_factor)
		setValue("blendfactor",blm.b)
		setValue("maxSubdiv",sm.max_subdiv)
		setValue("pixelThreshold",sm.pixel_threshold)
		setValue("curveThreshold",sm.curvature_threshold)
		setValue("igm",sm.linked_igm)
		###setValue("baseEmissionGain",sm.base_emission_gain)
		setValue("abs_map",absm.type)
		setValue("sss_map",sssm.type)
		setValue("Phase_Function",sm.phase_function)
		setValue("emissionScale",sm.emission_scale)
		setValue("emissionMeasure",sm.emission_measure)
		
		#set swatches
		setColorBox("PreviewImage",sm.mat.color.to_a)
		#set wavelength dependent param swatches
		setColorBox("absorption",sm.color("absorption"))
		setColorBox("baseEmissionColorBox",sm.color("base_emission"))
		setColorBox("emissionSwatch",sm.color("emission"))
		setColorBox("transmittanceSwatch",sm.color("transmittance"))
		setColorBox("albedoSwatch",sm.color("albedo"))
		setColorBox("abslayertransSwatch",sm.color("absorption_layer_transmittance"))
		setColorBox("specularSwatch",sm.color("specular"))
		setColorBox("sssSwatch",sm.color("sss"))
		setColorBox("g_spectrumSwatch",sm.color("g_spectrum"))
		
		#set value boxes visible or invisible
		setValueBoxesVisible()
		if sm.phase_function=="Uniform" then setVisible("g_spectrum_param",false)
		else setVisible("g_spectrum_param",true)
		end
		
		if sm.sss? then	setCheckbox("SSSCheckbox","true")
		else setCheckbox("SSSCheckbox","false")
		end
		#if  sm.smooth? then setCheckbox("subSmoothingCheckbox","true")
			#else setCheckbox("subSmoothingCheckbox","false")
		#end
		if  sm.view_dependent_subdivision? then setCheckbox("viewDependentCheckbox","true")
			else setCheckbox("viewDependentCheckbox","false")
		end
		if sm.ies? then setCheckbox("EnableIESCheckbox","true")
		else setCheckbox("EnableIESCheckbox","false")
		end
		if sm.use_nkdata? then setCheckbox("EnableNKdataCheckbox","true")
		else setCheckbox("EnableNKdataCheckbox","false")
		end
		if sm.use_bump? then setCheckbox("bumpCheckbox","true")
		else setCheckbox("bumpCheckbox","false")
		end
		if sm.use_normal? then setCheckbox("normalCheckbox","true")
		else setCheckbox("normalCheckbox","false")
		end
		if sm.use_displace? then setCheckbox("displaceCheckbox","true")
		else setCheckbox("displaceCheckbox","false")
		end
		if sm.use_emission? then setCheckbox("emissionCheckbox","true")
		else setCheckbox("emissionCheckbox","false")
		end
		if sm.use_abs_layer_trans? then setCheckbox("absLayerTransCheckbox","true")
		else setCheckbox("absLayerTransCheckbox","false")
		end
		if sm.use_clip? then setCheckbox("clipCheckbox","true")
		else setCheckbox("clipCheckbox","false")
		end
		if sm.use_specular? then setCheckbox("specularCheckbox","true")
		else setCheckbox("specularCheckbox","false")
		end
		if sm.step_blend? then setCheckbox("stepBlend","true")
		else setCheckbox("stepBlend","false")
		end
		if sm.interference? then setCheckbox("interferenceCheckbox","true")
		else setCheckbox("interferenceCheckbox","false")
		end
		if sm.arch_glass? then setCheckbox("archGlassCheckbox","true")
		else setCheckbox("archGlassCheckbox","false")
		end
		###if sm.phase_function=="Uniform" then setRadioButton("UniformPF","true")
		###else setRadioButton("Henyey-GreensteinPF","true")
		###end
		###if sm.base_emission_spectrum=="Blackbody" then setRadioButton("bbES","true")
		###else setRadioButton("rgbES","true")
		###end

		# update simple ui values
		if @ui=="simple"
			if type=="Diffuse"
				s_value="0"
				s2_value="100"
			elsif type=="Specular"
				s_value=to_reflection(sm.ior)
				s2_value="0"
			else
				s_value=to_reflection(sm.ior)
				s2_value=to_roughness(sm.exponent_map.b)
			end
			#s3_value=to_power_slider(sm.power_drawn)
			s3_value=to_power_slider(sm.emission_scale)
			
			cmd="s.setValue(#{s_value})"
			@mat_editor.execute_script(cmd)
			
			cmd="s2.setValue(#{s2_value})"
			@mat_editor.execute_script(cmd)
			
			cmd="s3.setValue(#{s3_value})"
			@mat_editor.execute_script(cmd)
		end
		if (type=="Specular") or (type=="Glossy Transparent") then setCheckbox("transparentCheckbox","true")
		else setCheckbox("transparentCheckbox","false")
		end
		if (sm.emitter?) then setCheckbox("emitterCheckbox","true")
		else setCheckbox("emitterCheckbox","false")
		end
	end

	#this method sets the visibility of the material editor UI elements based on the current material type
	def setVisibility()

		type=sm.type
		if @ui=="simple"
			cmd="setSimpleUIVisible()"
			@mat_editor.execute_script(cmd)
			return
		end
		case type
			when "Diffuse"
				@mat_editor.execute_script("setDiffuseVisible()")
			when "Phong"
				@mat_editor.execute_script("setPhongVisible()")
			when "Specular"
				@mat_editor.execute_script("setSpecularVisible()")
			when "Glossy Transparent"
				@mat_editor.execute_script("setGlossyTransparentVisible()")
			when "Blend"
				@mat_editor.execute_script("setBlendVisible()")
			when "Metals","Al","Cr","Cu","Au","Ag","Ti","W","Li","Ni","Pt"
				@mat_editor.execute_script("setMetalsVisible()")
			when "Thin Glass"
				@mat_editor.execute_script("setThinGlassVisible()")
			when "Oren-Nayar"
				@mat_editor.execute_script("setOrenNayarVisible()")
			when "Linked IGM"
				@mat_editor.execute_script("setLinkedIGMVisible()")
			when "RGB Light"
				@mat_editor.execute_script("setRGBLightVisible()")
			when "Blackbody Light"
				@mat_editor.execute_script("setBlackbodyLightVisible()")
			when "Exit Portal"
				@mat_editor.execute_script("setExitPortalVisible()")
			when "Coating"
				@mat_editor.execute_script("setCoatingVisible()")
			when "Double-Sided Thin"
				@mat_editor.execute_script("setDoubleSidedThinVisible()")
			when "Simple Thin"
				@mat_editor.execute_script("setSimpleThinVisible()")
		end
		@mat_editor.execute_script("setEfficacyVisible()")   #disables efficacy if ies profile is enabled
	end 
			
	def setValue(id,value)
		SkIndigo.set_value(@mat_editor, id, value)
	end
	def setCheckbox(id,checked)
		cmd="document.getElementById('#{id}').checked=#{checked}"
		@mat_editor.execute_script(cmd)
	end
	def setRadioButton(id,checked)
		cmd="document.getElementById('#{id}').checked=\"#{checked}\""
		@mat_editor.execute_script(cmd)
	end
	def setColorBox(id,color)

		if color!=nil
			r=color[0]
			g=color[1]
			b=color[2]
			cmd="$(\"##{id}\").show()"
			@mat_editor.execute_script(cmd)
			cmd="document.getElementById('#{id}').style.backgroundColor=\"rgb(#{r},#{g},#{b})\""
		else
			cmd="$(\"##{id}\").hide()"
		end
		@mat_editor.execute_script(cmd)

	end

	def setVisible(id,visible) #visible is boolean

		if visible==true
			cmd="$(\"##{id}\").show()"
		else
			cmd="$(\"##{id}\").hide()"
			#cmd="$(\"##{id}\").fadeTo('fast',0.3)"
		end
		@mat_editor.execute_script(cmd)
	end

	def setValueBoxesVisible()

		textures=[sm.bump_map,sm.exponent_map,sm.clip_map,sm.disp_map,sm.sigma_map,sm.blend_map,sm.normal_map,sm.roughness_map,sm.fresnel_map,sm.thickness_map,sm.front_roughness_map,sm.back_roughness_map,sm.front_fresnel_scale_map,sm.back_fresnel_scale_map,sm.r_f_map]
		dlg_ids=["bump_value","expmap_value","clipValue","dispmap_value","sigma_value","blendfactor","normalmap_value","roughness_value","fresnel_value","thickness_value","front_roughness_value","back_roughness_value","front_fresnel_value","back_fresnel_value","r_f_value"]

		textures.each_index {|i|
			setValueBoxVisible(textures[i],dlg_ids[i])
		}
	end

	def setValueBoxVisible(tex,id)

		if tex.type=="Shader"
			setVisible(id,false)
		else
			setVisible(id,true)
		end
	end

	
	def getTextures()  #Returns array of all materials with textures
		texture_list=["None","SU"]
		materials=Sketchup.active_model.materials
		materials.each do |m|
		if (m.respond_to?(:texture) and m.texture !=nil)
			texture_list.push(m.name)
		end
		next
		end
		texture_list+=["Custom Shader"]
		return texture_list
	end

	def getSubstrateList()  #return an array of all valid materials for substrates
		materials=Sketchup.active_model.materials
		list=[]
		for mat in materials
			type=IndigoMaterial.new(mat).type
			case type
				when "Blackbody Light","RGB Light","Exit Portal"
				###
				else
					list.push(mat.name) if mat!=sm.mat
			end
		end
		return list.sort.unshift("None")
	end

	def getBlendList() #Returns array of all valid materials for blending 
		materials=Sketchup.active_model.materials
		#blend_list=["None"]
		blend_list=[]
		for mat in materials
			type=IndigoMaterial.new(mat).type
			if ((type) !="Blackbody Light") and ((type)!="RGB Light") and ((type)!="Exit Portal")
				#blend_list.push(mat.display_name) if mat!=sm.mat
				blend_list.push(mat.name) if mat!=sm.mat
			end
		end 
		return blend_list.sort.unshift("None")
	end

	##############################Material Presets
	def apply_oj(material)
		material.type="Specular"
		material.ior=1.33
		material.cauchy_b_coeff=0.0
		material.sss=true
		material.sss_spectrum="250"
		material.alpha=0.41
		material.color=[255,231,0]
		material.phase_function="Henyey-Greenstein"
		material.g_spectrum="0.55"
		material.layer=-1
	end

	def apply_clay(mat)
		mat.type="Oren-Nayar"
		mat.alpha=1.0
		mat.layer=-1
		tex=mat.sigma_map
		tex.b=0.3
		mat.sigma_map=tex
		mat.sigma=0.3
		mat.color=[181, 158, 135]
	end

	def apply_wax(material)
		material.type="Specular"
		material.ior=1.1
		material.cauchy_b_coeff=0.0
		material.sss=true
		material.sss_spectrum="300"
		material.alpha=0.12
		material.color=[230,230,5]
		material.phase_function="Uniform"
		material.layer=-1
	end

	def apply_lambertian(material)
		material.type="Diffuse"
		material.alpha=1.0
		material.layer=-1
	end

	def apply_glass(material)
		material.type="Specular"
		material.ior=1.52
		material.cauchy_b_coeff=0.0
		material.sss=false
		material.alpha=0.0
		material.layer=-1
	end

	def apply_water(material)
		material.type="Specular"
		material.ior=1.33
		material.cauchy_b_coeff=0.0
		material.sss=true
		material.sss_spectrum="0.0007 0.002 0.0026"
		material.color=[26, 230, 246]
		material.alpha=0.02
		material.phase_function="Uniform"
		material.layer=-1
	end

	def apply_frosted_glass(material)
		material.type="Glossy Transparent"
		material.exponent=100
		material.ior=1.52
		material.cauchy_b_coeff=0.0
		material.sss=false
		material.alpha=0.0
		material.layer=-1
	end

	def apply_mirror(material)
		material.type="Specular"
		material.ior=20.0
		material.cauchy_b_coeff=0.0
		material.sss=false
		material.alpha=1.0
		material.layer=-1
	end

	def apply_ceramic(material)
		material.type="Phong"
		material.exponent=5000
		material.fresnel_scale=1.0
		material.ior=1.52
		material.alpha=1.0
		material.layer=-1
	end

	def apply_dull_plastic(material)
		material.type="Phong"
		material.exponent=300
		material.fresnel_scale=1.0
		material.ior=1.33
		material.alpha=1.0
		material.layer=-1
	end

	def apply_shiny_plastic(material)
		material.type="Phong"
		material.exponent=10000
		material.fresnel_scale=1.0
		material.ior=1.22
		material.alpha=1.0
		material.layer=-1
	end

	def apply_leather(material)
		material.type="Phong"
		material.exponent=50
		material.fresnel_scale=1.0
		material.ior=1.58
		material.alpha=1.0
		material.layer=-1
	end

	def apply_car_paint(material)
		material.type="Phong"
		material.exponent=30000
		material.fresnel_scale=1.0
		material.ior=1.72
		material.alpha=1.0
		material.layer=-1
	end

	def apply_tinted_glass(material)
		material.type="Specular"
		material.ior=1.52
		material.cauchy_b_coeff=0.0
		material.sss=false
		material.alpha=0.1
		material.layer=-1
	end

	def apply_nkdata(material,file_name)
		root=File.dirname(SkIndigo.get_indigo_path)
		return if not root
		nkdata=File.join(root,"nkdata",file_name)
		nkdata.gsub!(/\\/,"\/")
		nk=NKdata.new(nkdata)
		material.type="Metals"
		material.nk_data=nk
		material.use_nkdata=true
		material.layer=-1
	end

	def apply_lampshade(material)
		material.type="Diffuse"
		material.alpha=0.5
		material.layer=-1
	end

	def apply_100W(material)
		material.layer=0 if material.layer==-1
		material.emission_scale=1750.0
		material.emission_measure="luminous_flux"
		bm=material.base_emission
		bm.bb_temperature=2800
		bm.spectrum_type="Blackbody"
		material.base_emission=bm
		#material.base_emission_spectrum="Blackbody"
		@mat_editor.execute_script("document.getElementById('emitterTable').style.display='inline'")
	end

	def apply_60W(material)
		material.layer=0 if material.layer==-1
		material.emission_scale=60.0*14.0
		material.emission_measure="luminous_flux"
		material.base_emission_spectrum="Blackbody"
		@mat_editor.execute_script("document.getElementById('emitterTable').style.display='inline'")
	end

	def apply_7WCFW(material)
		material.layer=0 if material.layer==-1
		material.emission_scale=7.0*50.0
		material.emission_measure="luminous_flux"
		material.base_emission_spectrum="Blackbody"
		@mat_editor.execute_script("document.getElementById('emitterTable').style.display='inline'")
	end

	def apply_34WTFC(material)
		material.layer=0 if material.layer==-1
		material.emission_scale=34.0*50.0
		material.emission_measure="luminous_flux"
		bm=material.base_emission
		bm.bb_temperature=6000
		bm.spectrum_type="Blackbody"
		material.base_emission=bm
		@mat_editor.execute_script("document.getElementById('emitterTable').style.display='inline'")
	end

	def apply_34WTFM(material)
		material.layer=0 if material.layer==-1
		material.emission_scale=34.0*50.0
		material.emission_measure="luminous_flux"
		bm=material.base_emission
		bm.bb_temperature=4500
		bm.spectrum_type="Blackbody"
		material.base_emission=bm
		@mat_editor.execute_script("document.getElementById('emitterTable').style.display='inline'")
	end

	def apply_34WTFW(material)
		material.layer=0 if material.layer==-1
		material.emission_scale=34.0*50.0
		material.emission_measure="luminous_flux"
		bm=material.base_emission
		bm.bb_temperature=3000
		bm.spectrum_type="Blackbody"
		material.base_emission=bm
		@mat_editor.execute_script("document.getElementById('emitterTable').style.display='inline'")
	end

	###################calibration of simple UI sliders
	def to_ior(value)
		value=value.to_f
		specular=0.001*Math.exp(value*0.0692)
		new_ior=-(Math.sqrt(specular)+1)/(Math.sqrt(specular)-1)
		new_ior=(new_ior*1000).round.to_f/1000    #round the value to three decimal places
		return new_ior.to_s
	end
	def to_exp(value)
		value=value.to_i
		exp=1000000*Math.exp(-0.14*value)
		exp=exp.round
		return exp.to_s
	end
	def to_reflection(value)
		value=value.to_f
		specular=((value-1)/(value+1))**2
		if specular<=0.0
			reflection=0
		else
			reflection=Math.log(1000*specular)/0.0692
			reflection=100 if reflection>100
		end
		return reflection.to_s
	end
	def to_roughness(value)
		value=value.to_f
		rough=Math.log(value/1000000)/(-0.14)
		rough=100 if rough>100
		rough=0 if rough<0
		return rough.to_s
	end
	def to_power(value)
		value=value.to_i
		power=value*1
		return power.to_s
	end
	def to_power_slider(value)
		value=value.to_f
		power_slider=value/100.0
		power_slider=100 if power_slider>100
		power_slider=1 if power_slider<1
		power_slider=power_slider.to_i
		return power_slider.to_s
	end
	def getTypeFromSliders()
		current_type=sm.type
		ref=@mat_editor.get_element_value("reflection-value").to_i
		rough=@mat_editor.get_element_value("roughness-value").to_i
		opacity=sm.alpha
		if (current_type=="Blackbody Light") or (current_type=="RGB Light") or (current_type=="Metals") or (current_type=="Exit Portal") or (current_type=="Thin Glass") or (current_type=="Blend")
			return current_type
		elsif (current_type=="Specular") or (current_type=="Glossy Transparent")
			if rough==0
				type="Specular"
			else
				type="Glossy Transparent"
			end
		else
			if (rough==100) and (ref==0)
				type="Diffuse"
			else
				type="Phong"
			end
		end
		return type

	end

	def mat_editor    #material editor accessor function
		@mat_editor
	end

	def preset_dlg
		@prset_dlg
	end

	#################################################Material Preview
	def preview_material(mat, preview_model, thumbnail_path)

		use_console=true

		case File.basename(preview_model)
			when "IMPsphere.igs"
				if mat.emitter?
					UI.messagebox("The Sphere preview scene cannot be used for emitting materials.")
					return
				end
				if mat.type=="Linked IGM"
					UI.messagebox("The Sphere preview scene cannot be used for Linked IGM materials.")
					return
				end
		end

		igs_export_path=SkIndigo.default_export_path()

		on_mac=SkIndigo.on_mac?
		unless on_mac
			path=SkIndigo.get_indigo_console_path if use_console
			path=SkIndigo.get_indigo_path if not use_console
			indigo_dir=File.dirname(path)
			#preview_path=File.join(indigo_dir,"SUpreviews")
			#preview_path=File.join(Sketchup.find_support_file("Plugins\/skindigo"),"SUPreviews")
		else
			path=SkIndigo.get_indigo_console_path #currently using the indigo console for previews
			#path=SkIndigo.get_indigo_path
			#preview_path=File.join(Sketchup.find_support_file("Plugins\/skindigo"),"SUPreviews")
		end
		return if not path

		preview_igs_path=preview_model
		preview_mat_igs=File.join(igs_export_path,"previewmats.igs")

		if not FileTest.exist?(preview_igs_path)
			UI.messagebox "Preview IGS file not found in #{preview_igs_path}"
			return
		end

		#tex_path=File.join(preview_path,"textures")
		tex_path=File.join(igs_export_path,"textures")

		if not FileTest.exist? (tex_path)
			Dir.mkdir(tex_path)
		end
		ie=IndigoExporter.new(preview_mat_igs)
		#ie.tex_path=tex_path
		ie.tex_path="textures"
		ie.preview_material=mat

		ie.get_used_layers()
		#Write the new material preview file
		out = SkIndigo.new_writable_utf8_file(preview_mat_igs)
		#out = File.new(preview_path+"\\previewmats.igs",File::CREAT|File::TRUNC|File::RDWR, 0644)
		out.puts "<scene>"
		status=ie.export_material(mat,out)
		out.puts "<include>"
		out.puts "	<pathname>#{preview_igs_path}</pathname>"
		out.puts "</include>"
		out.puts "</scene>"
		out.close

		ie.export_textures(ie.textures)                     #exports the textures that were stored during the material export process
		#for nk_path in ie.nkdata					#exports the nkdata that was collected during the material export process
			#nk=NKdata.new(nk_path)
			#nk.export(tex_path)  #exports the nkdata to the textures directory
		#end


		out_image_path=thumbnail_path
		unless on_mac 
			params="-o \"#{out_image_path}\"" 
		else
			params="-o \"#{out_image_path}\""
		end
		SkIndigo.launch_indigo(path,preview_mat_igs,params,true,"hidden")

	end #function end

	
	
	
	######
	def stop_preview()

		begin
			if @on_mac
				if @preview_process_id
					system("kill #{@preview_process_id}")
				end
			else
				if @preview_process_id
					# puts "Killing process, @preview_process_id: #{@preview_process_id}"
					Process.kill(9, @preview_process_id.to_i)
				end

			end

			#now kill the thumbnail refresh timer
			if @timer
				UI.stop_timer(@timer)
			end

			@preview_process_id=nil
			toggle_stop_button(@mat_editor,false)
		rescue
			raise
		end
	end #function

	
	
	
	#######dlg is the webdialog, id is the HTML id, status is either true or false
	def toggle_stop_button(dlg,status)

		if status  #enable the button
			cmd="document.getElementById('StopPreview').disabled=\"\""
			cmd2="document.getElementById('StopButtonImage').src=\"player_stop.png\""
		else #disable the button
			cmd="document.getElementById('StopPreview').disabled=\"disabled\""
			cmd2="document.getElementById('StopButtonImage').src=\"player_stop_disabled.png\""
		end
		dlg.execute_script(cmd)
		dlg.execute_script(cmd2)
	end


	#####
	def preview_running?
		wmi = WIN32OLE.connect("winmgmts://")
		query = "select * from Win32_process"
		processes = wmi.ExecQuery(query)
		names=[]
		processes.each{|p| names.push(p.Name)}
		return true if names.include?("indigo_console.exe")
	end #function

	
	#####
	def download_igm(url,dl_path=nil)

		igm_filename = File.basename(url)
		tempdir = SkIndigo.temp_dir
		dl_path = File.join(tempdir,igm_filename) unless dl_path
		
		@download_dialog = UI::WebDialog.new("Downloading material", false, "DownloadingDialog3", 
			450, 150, # width, height
			400,300, # x, y
			true)
		fn = File.join(File.dirname(reinterpret_as_utf8(__FILE__)), "Downloading.html")
		@download_dialog.set_file fn
		@download_dialog.show_modal do
			SkIndigo.download_file(url, dl_path)
			@download_dialog.close()
		end
		
		

		if FileTest.exist?(dl_path)
			return dl_path
		else
			return nil
		end
	end  #end functin

end #material editor class
